// Map.c - GBA Map Handling Functions
// ---------------------------------------------------
// Copyright (C)Savage Development.
// By Matthew Tighe
// ***************************************************
// ***************************************************

#include "..\gba.h"
#include "..\savage.h"

// mapPut( x, y, width, height, pDest, pSource, start)
// ---------------------------------------------------
// Puts a linear rectangular region into the specified
// place in map <pDest>

void mapPut( int x, int y, int width, int height, u16 *pDest, u16 *pSrc, u16 start)
{
	int i,j;
	u32 skew;
		
	pDest += ( (y * 32) + x );
	skew = ( 32 - width );

	for(j = 0; j < height; j++)
	{
		for(i = 0; i < width; i++)
			*pDest++ = (start + *pSrc++);

		pDest += skew;
	}
}

// mapSet( x, y, width, height, pDest, tile)
// ---------------------------------------------------
// Puts a rectangle of <tile> into the specified
// place in map <pDest>

void mapSet( int x, int y, int width, int height, u16 *pDest, u16 tile)
{
	int i,j;
	u32 skew;
		
	pDest += ( (y * 32) + x );
	skew = ( 32 - width );

	for(j = 0; j < height; j++)
	{
		for(i = 0; i < width; i++)
			*pDest++ = tile;

		pDest += skew;
	}
}

// mapPutLinear( x, y, width, height, pDest, start)
// ---------------------------------------------------
// Puts a rectangular region starting with <start>
// and increasing every block at a place in map <pDest>

void mapPutLinear( int x, int y, int width, int height, u16 *pDest, u16 start)
{
	int i,j;
	u32 skew;
		
	pDest += ( (y * 32) + x );
	skew = ( 32 - width );

	for(j = 0; j < height; j++)
	{
		for(i = 0; i < width; i++)
			*pDest++ = start++;

		pDest += skew;
	}
}

// mapPrint( x, y, pMap, szText, pFont)
// ---------------------------------------------------
// Puts a string of text onto the map in a specified font

void mapPrint( int x, int y, u16 *pMap, char *szText, FONT *pFont)
{
	int i;
	u16 dwChar;

	i = 0;

	pMap += ( (y * 32) + x );

	do
	{
		dwChar = ( u8 ) ( szText[i] );

		pMap[i] = dwChar + pFont->dwAdjust;
		i++;

	} while( szText[i] != '\0' );
}
