// Savage.h - Main Include File
// -------------------------------------------
// Savage Graphics System  for Gameboy Advance
// Copyright (C)Savage Development
// By Matthew Tighe

typedef struct _gfxSprite
{
	BYTE type;			// 0 - Type of sprite
	BYTE alpha;			// 1 - Alpha value
	BYTE size;			// 2 - Size 4bits
	BYTE priority;		// 3 - Priority with backgrounds

	WORD x;				// 4
	WORD y;				// 6
	WORD z;				// 8
	WORD tile;			// 10 - Tile offset in VRAM
	WORD flags;			// 12
	WORD rotscale;		// 14
	
	BYTE *pData;		// 16 - Pointer to RAM data (if any)
	WORD *pAttach;		// 20 - X,Y Attachment pointer
	void *pPrev;		// 24
	void *pNext;		// 28
						// 32 Bytes
} GFXSPRITE;

typedef struct _gfxList
{

	WORD number;		// 0 - Number of objects in list, not including header sprite
	WORD pad;			// 2
	GFXSPRITE *pHead;	// 4 - Pointer to header sprite
						// 8 Bytes

} GFXLIST;

typedef struct gbaResourceHeader
{
	BYTE dbType;		// Resource Type
	BYTE dbFormat;		// Resource Format Flags
	WORD dwSize;		// Header size including extra header.
	DWORD dbLength;		// Data length

} RESHEADER;

typedef struct gbaBlockHeader
{
	BYTE dbType;		// Resource Type
	BYTE dbFormat;		// Resource Format Flags
	WORD dwSize;		// Header size including extra header
	DWORD ddLength;

	WORD dwWidth;		// Width of block (pixels)
	WORD dwHeight;		// Height
	WORD dwNumber;		// Number of blocks in resource
	WORD dwFlags;		// Flags ie colour depth

} BLKHEADER;

typedef struct gbaMapHeader
{
	BYTE dbType;		// Resource Type
	BYTE dbFormat;		// Resource Format Flags
	WORD dwSize;		// Header size including extra header.
	DWORD ddLength;

	WORD dwWidth;		// Width of map (blocks)
	WORD dwHeight;		// Height
	WORD dwBlockSize;	// Size of blocks
	WORD dwFlags;		// Flags ie colour depth

} MAPHEADER;

typedef struct gbaPalHeader
{
	BYTE dbType;		// Resource Type
	BYTE dbFormat;		// Resource Format Flags
	WORD dwSize;		// Header size including extra header.
	DWORD ddLength;		// Data length

	WORD dwColours;		// Number of colours in palette.

} PALHEADER;

typedef struct vramBlock
{
	DWORD ddFlags;
	DWORD ddLength;
	void *pAddress;
	struct vramBlock *pPrev;
	struct vramBlock *pNext;

} VRAMBLOCK;

#define VRAM_ALLOC_OBJ 1
#define VRAM_BLOCK_USED 0x10000

#define VRAM_ALIGN_MAP 2048		// Map granularity is 2 KBytes
#define VRAM_ALIGN_CHAR 16384	// Tile granularity is 16 KBytes

#define RES_TYPE_BIN 0
#define RES_TYPE_BLOCKS 1
#define RES_TYPE_MAP 2
#define RES_TYPE_PALETTE 3

#define RES_FORMAT_RAW 0
#define RES_FORMAT_RLE 1

#define RES_PALLTTE_256 0
#define RES_PALETTE_16 1

#define SPR_8x8		0
#define SPR_16x16	1
#define SPR_32x32	2
#define SPR_64x64	3
#define SPR_16x8	4
#define SPR_32x8	5
#define SPR_32x16	6
#define SPR_64x32	7
#define SPR_8x16	8
#define SPR_8x32	9
#define SPR_16x32	10
#define SPR_32x64	11

#define SPRDRAW_ZORDER 1		// Draw sprites with Z-Order

// font structure, describes a font in character vram
struct font
{
	u16	dwStart;			// Starting char num in vram
	s16 dwAdjust;			// Adjustment for chars not in font
	u16 dwHeight;			// Height of font in chars
	u16 dwWidth;			// Width of font in chars
};

typedef struct font FONT;

// Map Handling Functions
// ----------------------
void mapPrint( int x, int y, u16 *pMap, char *szText, FONT *pFont);
void mapSet( int x, int y, int width, int height, u16 *pDest, u16 tile);
void mapPut( int x, int y, int width, int height, u16 *pDest, const u16 *pSrc, u16 start);
void mapPutLinear( int x, int y, int width, int height, u16 *pDest, u16 start);

// Sprite Handling Functions
// -------------------------
void sprInit( GFXLIST *pList);
GFXSPRITE *sprCreate(GFXLIST *pList);
void sprDestroy(GFXLIST *pList, GFXSPRITE *pSpr);
void sprDraw(GFXLIST *pList, u32 uFlags);
void sprSort(GFXLIST *pList);
void sprAttach(GFXSPRITE *pSlave, GFXSPRITE *pMaster);
void sprSwap(GFXSPRITE *pDest, GFXSPRITE *pSource );
void oamClear( int nStart, int nCount );


// Vram Allocation
// -------------------------
VRAMBLOCK *vramInit(void *pBase, DWORD ddLength);
void *vramAlloc(DWORD ddSize, DWORD ddAlign, WORD dwFlags);
VRAMBLOCK *vramUseBlock(VRAMBLOCK *pBlock, DWORD ddSize, DWORD ddPad);
DWORD CalcPadding(VRAMBLOCK *pBlock, DWORD ddSize, DWORD ddAlign);


// Globals
// -------------------------

u16 padStatus;					// Holds last joypad value